@@ -14,7 +14,7 @@ import warnings
 import jsonlines
 
 from .data_transfer import (
-    calculate_sha256, copy_file, copy_file_list, get_bytes, get_size_and_meta,
+    calculate_sha256, copy_file, copy_file_list, get_bytes, get_size_and_version,
     list_object_versions, put_bytes
 )
 from .exceptions import PackageException
@@ -188,7 +188,7 @@ class PackageEntry(object):
             when deserialization metadata is not present
         """
         physical_key = _to_singleton(self.physical_keys)
-        data, _ = get_bytes(physical_key)
+        data = get_bytes(physical_key)
 
         if func is not None:
             return func(data)
@@ -222,7 +222,7 @@ class PackageEntry(object):
         else:
             dest = fix_url(dest)
 
-        copy_file(physical_key, dest, self._meta)
+        copy_file(physical_key, dest)
 
         # return a package reroot package physical keys after the copy operation succeeds
         # see GH#388 for context
@@ -338,7 +338,7 @@ class Package(object):
             if registry is None:
                 raise QuiltException(
                     "No registry specified and no default_remote_registry configured. Please "
-                    "specify a registry or configure a default remote registry with quilt.config"
+                    "specify a registry or configure a default remote registry with quilt3.config"
                 )
 
         if dest_registry is None:
@@ -383,23 +383,20 @@ class Package(object):
         """
         if registry is None:
             registry = get_from_config('default_local_registry')
+        else:
+            registry = fix_url(registry)
 
         registry = registry.rstrip('/')
         validate_package_name(name)
-        name = quote(name)
 
-        if top_hash is not None:
-            # TODO: verify that name is correct with respect to this top_hash
-            # TODO: allow partial hashes (e.g. first six alphanumeric)
-            pkg_manifest_uri = fix_url(f'{registry}/.quilt/packages/{top_hash}')
-            return cls._from_path(pkg_manifest_uri)
-        else:
-            pkg_timestamp_file = f'{registry}/.quilt/named_packages/{name}/latest'
-            latest_pkg_hash, _ = get_bytes(pkg_timestamp_file)
-            latest_pkg_hash = latest_pkg_hash.decode('utf-8').strip()
-            pkg_manifest_uri = fix_url(f'{registry}/.quilt/packages/{quote(latest_pkg_hash)}')
-            return cls._from_path(pkg_manifest_uri)
+        if top_hash is None:
+            top_hash_url = f'{registry}/.quilt/named_packages/{quote(name)}/latest'
+            top_hash = get_bytes(top_hash_url).decode('utf-8').strip()
 
+        # TODO: verify that name is correct with respect to this top_hash
+        # TODO: allow partial hashes (e.g. first six alphanumeric)
+        pkg_manifest_uri = f'{registry}/.quilt/packages/{quote(top_hash)}'
+        return cls._from_path(pkg_manifest_uri)
 
     @classmethod
     def _from_path(cls, uri):
@@ -409,7 +406,7 @@ class Package(object):
             with open(parse_file_url(src_url)) as open_file:
                 pkg = cls.load(open_file)
         elif src_url.scheme == 's3':
-            body, _ = get_bytes(uri)
+            body = get_bytes(uri)
             pkg = cls.load(io.BytesIO(body))
         else:
             raise NotImplementedError
@@ -478,7 +475,7 @@ class Package(object):
             physical_key = _to_singleton(entry.physical_keys)
             new_physical_key = f'{nice_dest}/{quote(logical_key)}'
 
-            file_list.append((physical_key, new_physical_key, entry.size, entry.meta))
+            file_list.append((physical_key, new_physical_key, entry.size))
 
             # return a package reroot package physical keys after the copy operation succeeds
             # see GH#388 for context
@@ -646,7 +643,7 @@ class Package(object):
 
         return self
 
-    def get(self, logical_key=None):
+    def get(self, logical_key):
         """
         Gets object from logical_key and returns its physical path.
         Equivalent to self[logical_key].get().
@@ -661,78 +658,10 @@ class Package(object):
             KeyError: when logical_key is not present in the package
             ValueError: if the logical_key points to a Package rather than PackageEntry.
         """
-        if logical_key:
-            obj = self[logical_key]
-            if not isinstance(obj, PackageEntry):
-                raise ValueError("Key does not point to a PackageEntry")
-            return obj.get()
-        else:
-            # a package has a logical root directory if all of its children are rooted at the
-            # same directory or object path. in other words, the following things must match:
-            # * the URL scheme of every physical key is the same
-            # * the root path to each physical key is the same
-            first_lkey, first_entry = next(self.walk())
-            first_pkey = first_entry.physical_keys[0].split('?versionId=')[0]
-            hypothesized_scheme = urlparse(first_pkey).scheme
-
-            # ensure that the first entry has a logically consistent physical and logical key
-            if not first_pkey.endswith(first_lkey):
-                raise QuiltException(
-                    f"Cannot get the root directory for this package because it is not "
-                    f"physically consistent, as it contains entries whose logical keys and "
-                    f"physical keys have different names. For example, the {first_lkey!r} package "
-                    f"entry points to a file named {first_pkey.split('/')[-1]!r} (expected a file "
-                    f"named {first_lkey.split('/')[-1]!r}). To make this package physically "
-                    f"consistent run 'quilt.Package.install(\"$PKG_NAME\")', replacing '$PKG_NAME' "
-                    f"with the name of this package. Note that this will result in file copying."
-                )
-            hypothesized_root_path = first_pkey[:first_pkey.rfind(first_lkey)]
-
-            # every subsequent entry will be checked to ensure that its logical key is rooted
-            # at the same physical key as the first entry
-            for lkey, entry in self.walk():
-                pkey = entry.physical_keys[0].split('?versionId=')[0]
-                scheme = urlparse(pkey).scheme
-                root_path = pkey[:pkey.rfind(lkey)]
-
-                if scheme != hypothesized_scheme:
-                    raise QuiltException(
-                        f"Cannot get the root directory for this package because it is not "
-                        f"physically consistent, as it contains both local and remote entries. "
-                        f"For example, the {first_lkey!r} package entry is a "
-                        f"'{hypothesized_scheme}' entry, whist the {lkey!r} package entry is a "
-                        f"'{scheme}' entry. To make this package physically "
-                        f"consistent run 'quilt.Package.install(\"$PKG_NAME\")', replacing "
-                        f"'$PKG_NAME' with the name of this package. Note that this will result "
-                        f"in file copying."
-                    )
-                elif not pkey.endswith(lkey):
-                    raise QuiltException(
-                        f"Cannot get the root directory for this package because it is not "
-                        f"physically consistent, as it contains entries whose logical keys and "
-                        f"physical keys have different names. For example, the "
-                        f"{lkey!r} package entry points to a file named "
-                        f"{pkey.split('/')[-1]!r} (expected a file named "
-                        f"{first_lkey.split('/')[-1]!r}). To make this package physically "
-                        f"consistent run 'quilt.Package.install(\"$PKG_NAME\")', replacing "
-                        f"'$PKG_NAME' with the name of this package. Note that this will result "
-                        f"in file copying."
-                    )
-                elif root_path != hypothesized_root_path:
-                    raise QuiltException(
-                        f"Cannot get the root directory for this package because it is not "
-                        f"physically consistent, as it contains entries rooted in different "
-                        f"physical locations. For example, the {lkey!r} entry is located in the "
-                        f"{'/'.join(pkey.split('/')[:-1])!r} directory, whilst the "
-                        f"{first_lkey!r} is located in the "
-                        f"{'/'.join(first_pkey.split('/')[:-1])!r} directory. To make this "
-                        f"package physically consistent run "
-                        f"'quilt.Package.install(\"$PKG_NAME\")', "
-                        f"replacing '$PKG_NAME' with the name of this package. Note that this "
-                        f"will result in file copying."
-                    )
-
-            return hypothesized_root_path
+        obj = self[logical_key]
+        if not isinstance(obj, PackageEntry):
+            raise ValueError("Key does not point to a PackageEntry")
+        return obj.get()
 
     # def get_as_pathlib(self):
 
@@ -797,22 +726,23 @@ class Package(object):
 
         if registry is None:
             registry = get_from_config('default_local_registry')
+        else:
+            registry = fix_url(registry)
 
         registry = registry.rstrip('/')
         validate_package_name(name)
-        name = quote(name)
 
         self._fix_sha256()
         manifest = io.BytesIO()
         self.dump(manifest)
 
-        pkg_manifest_file = f'{registry}/.quilt/packages/{self.top_hash}'
+        pkg_manifest_file = f'{registry}/.quilt/packages/{quote(self.top_hash)}'
         put_bytes(
             manifest.getvalue(),
             pkg_manifest_file
         )
 
-        named_path = f'{registry}/.quilt/named_packages/{name}/'
+        named_path = f'{registry}/.quilt/named_packages/{quote(name)}/'
         # TODO: use a float to string formater instead of double casting
         hash_bytes = self.top_hash.encode('utf-8')
         timestamp_path = named_path + str(int(time.time()))
@@ -890,7 +820,7 @@ class Package(object):
 
         if isinstance(entry, (str, os.PathLike)):
             url = fix_url(str(entry))
-            size, orig_meta, version = get_size_and_meta(url)
+            size, version = get_size_and_version(url)
 
             # Determine if a new version needs to be appended.
             parsed_url = urlparse(url)
@@ -898,7 +828,7 @@ class Package(object):
                 bucket, key, current_version = parse_s3_url(parsed_url)
                 if not current_version and version:
                     url = make_s3_url(bucket, key, version)
-            entry = PackageEntry([url], size, None, orig_meta)
+            entry = PackageEntry([url], size, None, None)
         elif isinstance(entry, PackageEntry):
             entry = entry._clone()
 
@@ -1051,7 +981,7 @@ class Package(object):
 
         if registry is None:
             registry = get_from_config('default_remote_registry')
-            if not registry:
+            if registry is None:
                 raise QuiltException(
                     "No registry specified and no default remote registry configured. Please "
                     "specify a registry or configure a default remote registry with quilt3.config"
@@ -1119,6 +1049,28 @@ class Package(object):
         pkg.build(name, registry=registry, message=message)
         return pkg
 
+    @classmethod
+    def rollback(cls, name, registry, top_hash):
+        """
+        Set the "latest" version to the given hash.
+
+        Args:
+            name(str): Name of package to rollback.
+            registry(str): Registry where package is located.
+            top_hash(str): Hash to rollback to.
+        """
+        registry = fix_url(registry).rstrip('/')
+        validate_package_name(name)
+
+        hash_path = f'{registry}/.quilt/packages/{quote(top_hash)}'
+        latest_path = f'{registry}/.quilt/named_packages/{quote(name)}/latest'
+
+        # Check that both latest and top_hash actually exist.
+        get_size_and_version(hash_path)
+        get_size_and_version(latest_path)
+
+        put_bytes(top_hash.encode('utf-8'), latest_path)
+
     def _materialize(self, dest_url):
         """
         Copies all Package entries to the destination, then creates a new package that points to those objects.
@@ -1153,7 +1105,7 @@ class Package(object):
                 pkg.set(logical_key, new_entry)
             else:
                 entries.append((logical_key, new_entry))
-                file_list.append((physical_key, new_physical_key, entry.size, None))
+                file_list.append((physical_key, new_physical_key, entry.size))
 
         results = copy_file_list(file_list)
 
