@@ -18,7 +18,9 @@ APP_NAME = "Quilt"
 APP_AUTHOR = "QuiltData"
 BASE_DIR = user_data_dir(APP_NAME, APP_AUTHOR)
 BASE_PATH = pathlib.Path(BASE_DIR)
+TEMPFILE_DIR_PATH = BASE_PATH / "tempfiles"
 CONFIG_PATH = BASE_PATH / 'config.yml'
+OPEN_DATA_URL = "https://open.quiltdata.com"
 
 PACKAGE_NAME_FORMAT = r"[\w-]+/[\w-]+$"
 
@@ -95,6 +97,23 @@ def fix_url(url):
     return fixed_url
 
 
+def extract_file_extension(file_path_or_url):
+    """
+    Extract the file extension if it exists.
+
+    Args:
+        file_path_or_url: The path to the file. Type can can be anything that pathlib.Path understands.
+
+    Returns:
+        File extension without the period, i.e. ("txt" not ".txt"). None if the path does not have an extension.
+    """
+    p = pathlib.Path(file_path_or_url)
+    if len(p.suffix) > 0:
+        return p.suffix[1:]
+    else:
+        return None
+
+
 EXAMPLE = "Example: 's3://my-bucket/path/'."
 def parse_s3_url(s3_url):
     """
@@ -136,6 +155,8 @@ def parse_file_url(file_url):
         path = '\\\\%s%s' % (file_url.netloc, path)
     return path
 
+def file_is_local(file_url_or_path):
+    return urlparse(fix_url(file_url_or_path)).scheme == 'file'
 
 def read_yaml(yaml_stream):
     yaml = ruamel.yaml.YAML()
@@ -250,7 +271,7 @@ class QuiltConfig(OrderedDict):
 
     # TODO: Make an _html_repr_ for nicer Notebook display
     def __repr__(self):
-        return "Configured to {}".format(self['navigator_url'])
+        return "<{} at {!r} {}>".format(type(self).__name__, str(self.filepath), json.dumps(self, indent=4))
 
 
 def find_bucket_config(bucket_name, catalog_config_url):
@@ -262,7 +283,7 @@ def find_bucket_config(bucket_name, catalog_config_url):
         federations = config_json['federations']
     else:
         registry_url = config_json['registryUrl']
-        federations = ["{reg_url}/api/federation.json".format(reg_url=registry_url)]
+        federations = ["{reg_url}/api/buckets".format(reg_url=registry_url)]
 
     federations.reverse() # want to get results from last federation first
     for federation in federations:
@@ -289,9 +310,9 @@ def find_bucket_config(bucket_name, catalog_config_url):
     raise QuiltException("Failed to find a config for the chosen bucket")
 
 def validate_package_name(name):
-    """ Verify that a package name is two alphanumerics strings separated by a slash."""
+    """ Verify that a package name is two alphanumeric strings separated by a slash."""
     if not re.match(PACKAGE_NAME_FORMAT, name):
-        raise QuiltException("Invalid package name, must contain exactly one /.")
+        raise QuiltException(f"Invalid package name: {name}.")
 
 def get_package_registry(path=None):
     """ Returns the package registry root for a given path """
@@ -299,13 +320,41 @@ def get_package_registry(path=None):
         path = get_from_config('default_local_registry')
     return path.rstrip('/') + '/.quilt'
 
+def configure_from_url(catalog_url):
+    """ Read configuration settings from a Quilt catalog """
+    config_template = read_yaml(CONFIG_TEMPLATE)
+    # Clean up and validate catalog url
+    catalog_url = catalog_url.rstrip('/')
+    validate_url(catalog_url)
+
+    # Get the new config
+    config_url = catalog_url + '/config.json'
+
+    response = requests.get(config_url)
+    if not response.ok:
+        message = "An HTTP Error ({code}) occurred: {reason}"
+        raise QuiltException(
+            message.format(code=response.status_code, reason=response.reason),
+            response=response
+            )
+    # QuiltConfig may perform some validation and value scrubbing.
+    new_config = QuiltConfig('', response.json())
+
+    # 'navigator_url' needs to be renamed, the term is outdated.
+    if not new_config.get('navigator_url'):
+        new_config['navigator_url'] = catalog_url
+
+    # Use our template + their configured values, keeping our comments.
+    for key, value in new_config.items():
+        config_template[key] = value
+    write_yaml(config_template, CONFIG_PATH, keep_backup=True)
+    return config_template
+
 def load_config():
     # For user-facing config, use api.config()
-    if CONFIG_PATH.exists():
-        local_config = read_yaml(CONFIG_PATH)
-    else:
-        config_template = read_yaml(CONFIG_TEMPLATE)
-        local_config = config_template
+    if not CONFIG_PATH.exists():
+        configure_from_url(OPEN_DATA_URL)
+    local_config = read_yaml(CONFIG_PATH)
     return local_config
 
 def get_from_config(key):
