@@ -37,7 +37,6 @@ def mock_make_api_call(self, operation_name, kwarg):
     if operation_name == 'HeadObject':
         # TODO: mock this somehow
         parsed_response = {
-            'Metadata': {},
             'ContentLength': 0
         }
         return parsed_response
@@ -173,7 +172,7 @@ class PackageTest(QuiltTestCase):
             pkgmock.reset_mock()
 
             with patch('quilt3.packages.get_bytes') as dl_mock:
-                dl_mock.return_value = (top_hash.encode('utf-8'), None)
+                dl_mock.return_value = top_hash.encode('utf-8')
                 pkg = Package.browse('Quilt/nice-name')
                 assert registry + '/.quilt/named_packages/Quilt/nice-name/latest' \
                         == dl_mock.call_args_list[0][0][0]
@@ -194,7 +193,7 @@ class PackageTest(QuiltTestCase):
 
             pkgmock.reset_mock()
             with patch('quilt3.packages.get_bytes') as dl_mock:
-                dl_mock.return_value = (top_hash.encode('utf-8'), None)
+                dl_mock.return_value = top_hash.encode('utf-8')
                 pkg = Package.browse('Quilt/nice-name', registry=remote_registry)
             assert '{}/.quilt/packages/{}'.format(remote_registry, top_hash) \
                     in [x[0][0] for x in pkgmock.call_args_list]
@@ -305,7 +304,7 @@ class PackageTest(QuiltTestCase):
              .set('foo', os.path.join(os.path.dirname(__file__), 'data', 'foo.txt'))['foo']
              .fetch())
             filepath = fix_url(os.path.join(os.path.dirname(__file__), 'data', 'foo.txt'))
-            copy_mock.assert_called_once_with(filepath, ANY, ANY)
+            copy_mock.assert_called_once_with(filepath, ANY)
 
     def test_load_into_quilt(self):
         """ Verify loading local manifest and data into S3. """
@@ -320,7 +319,6 @@ class PackageTest(QuiltTestCase):
                 'Body': ANY,
                 'Bucket': 'my_test_bucket',
                 'Key': 'Quilt/package/foo1',
-                'Metadata': {'helium': '{}'}
             }
         )
 
@@ -333,7 +331,6 @@ class PackageTest(QuiltTestCase):
                 'Body': ANY,
                 'Bucket': 'my_test_bucket',
                 'Key': 'Quilt/package/foo2',
-                'Metadata': {'helium': '{}'}
             }
         )
 
@@ -346,7 +343,6 @@ class PackageTest(QuiltTestCase):
                 'Body': ANY,
                 'Bucket': 'my_test_bucket',
                 'Key': '.quilt/packages/' + top_hash1,
-                'Metadata': {'helium': 'null'}
             }
         )
 
@@ -359,7 +355,6 @@ class PackageTest(QuiltTestCase):
                 'Body': top_hash1.encode(),
                 'Bucket': 'my_test_bucket',
                 'Key': '.quilt/named_packages/Quilt/package/1234567890',
-                'Metadata': {'helium': 'null'}
             }
         )
 
@@ -372,7 +367,6 @@ class PackageTest(QuiltTestCase):
                 'Body': top_hash1.encode(),
                 'Bucket': 'my_test_bucket',
                 'Key': '.quilt/named_packages/Quilt/package/latest',
-                'Metadata': {'helium': 'null'}
             }
         )
 
@@ -386,7 +380,7 @@ class PackageTest(QuiltTestCase):
         new_pkg.set('foo2', test_file1)
 
         with patch('time.time', return_value=1234567890), \
-             patch('quilt3.data_transfer.s3_threads', 1):
+             patch('quilt3.data_transfer.s3_transfer_config.max_request_concurrency', 1):
             remote_pkg = new_pkg.push('Quilt/package', 's3://my_test_bucket/')
 
         # Modify one file, and check that only that file gets uploaded.
@@ -401,7 +395,6 @@ class PackageTest(QuiltTestCase):
                 'Body': ANY,
                 'Bucket': 'my_test_bucket',
                 'Key': 'Quilt/package/foo2',
-                'Metadata': {'helium': '{}'}
             }
         )
 
@@ -414,7 +407,6 @@ class PackageTest(QuiltTestCase):
                 'Body': ANY,
                 'Bucket': 'my_test_bucket',
                 'Key': '.quilt/packages/' + top_hash2,
-                'Metadata': {'helium': 'null'}
             }
         )
 
@@ -427,7 +419,6 @@ class PackageTest(QuiltTestCase):
                 'Body': top_hash2.encode(),
                 'Bucket': 'my_test_bucket',
                 'Key': '.quilt/named_packages/Quilt/package/1234567891',
-                'Metadata': {'helium': 'null'}
             }
         )
 
@@ -440,7 +431,6 @@ class PackageTest(QuiltTestCase):
                 'Body': top_hash2.encode(),
                 'Bucket': 'my_test_bucket',
                 'Key': '.quilt/named_packages/Quilt/package/latest',
-                'Metadata': {'helium': 'null'}
             }
         )
 
@@ -449,7 +439,7 @@ class PackageTest(QuiltTestCase):
         remote_pkg.set('foo2', test_file3)
 
         with patch('time.time', return_value=1234567891), \
-             patch('quilt3.data_transfer.s3_threads', 1):
+             patch('quilt3.data_transfer.s3_transfer_config.max_request_concurrency', 1):
             remote_pkg.push('Quilt/package', 's3://my_test_bucket/')
 
 
@@ -598,9 +588,10 @@ class PackageTest(QuiltTestCase):
         """Verify that list returns packages in the appdirs directory."""
 
         # Build a new package into the local registry.
-        Package().build("Quilt/Foo")
-        Package().build("Quilt/Bar")
-        Package().build("Quilt/Test")
+        with patch('time.time', return_value=1234567890):
+            Package().build("Quilt/Foo")
+            Package().build("Quilt/Bar")
+            Package().build("Quilt/Test")
 
         # Verify packages are returned.
         pkgs = list(quilt3.list_packages())
@@ -608,6 +599,12 @@ class PackageTest(QuiltTestCase):
         assert "Quilt/Foo" in pkgs
         assert "Quilt/Bar" in pkgs
 
+        versions = set(quilt3.list_package_versions('Quilt/Foo'))
+        assert versions == {
+            ('latest', '2a5a67156ca9238c14d12042db51c5b52260fdd5511b61ea89b58929d6e1769b'),
+            ('1234567890', '2a5a67156ca9238c14d12042db51c5b52260fdd5511b61ea89b58929d6e1769b'),
+        }
+
         # Verify specifying a local path explicitly works as expected.
         assert list(pkgs) == list(quilt3.list_packages(LOCAL_REGISTRY.as_posix()))
 
@@ -993,7 +990,7 @@ class PackageTest(QuiltTestCase):
         assert repr(pkg) == TEST_REPR
 
     def test_remote_repr(self):
-        with patch('quilt3.packages.get_size_and_meta', return_value=(0, dict(), '0')):
+        with patch('quilt3.packages.get_size_and_version', return_value=(0, '0')):
             TEST_REPR = (
                 "(remote Package)\n"
                 " └─asdf\n"
@@ -1112,51 +1109,26 @@ class PackageTest(QuiltTestCase):
         with pytest.raises(QuiltException):
             pkg.set('s3://foo/..', LOCAL_MANIFEST)
 
+    def test_rollback(self):
+        p = Package()
+        p.set('foo', DATA_DIR / 'foo.txt')
+        p.build('quilt/tmp')
 
-    def test_default_package_get_local(self):
-        foodir = pathlib.Path("foo_dir")
-        bazdir = pathlib.Path("baz_dir")
-        foodir.mkdir(parents=True, exist_ok=True)
-        bazdir.mkdir(parents=True, exist_ok=True)
-        with open('bar', 'w') as fd:
-            fd.write(fd.name)
-        with open('foo', 'w') as fd:
-            fd.write(fd.name)
-        with open(bazdir / 'baz', 'w') as fd:
-            fd.write(fd.name)
-        with open(foodir / 'bar', 'w') as fd:
-            fd.write(fd.name)
+        good_hash = p.top_hash
 
-        currdir = pathlib.Path('.').resolve().as_uri() + '/'
+        assert 'foo' in Package.browse('quilt/tmp')
 
-        # consistent local case
-        pkg = quilt3.Package().set_dir("/", "./")
-        assert pkg.get() == currdir
+        p.delete('foo')
+        p.build('quilt/tmp')
 
-        # package with one inconsistent path, leading case
-        pkg = quilt3.Package().set_dir("/", "./")
-        pkg.set('badpath', 'bar')
-        with pytest.raises(QuiltException):
-            pkg.get()
+        assert 'foo' not in Package.browse('quilt/tmp')
 
-        # package with one inconsistent path, training case
-        pkg = quilt3.Package().set_dir("/", "./")
-        # prefix with 'z_' to ensure that this entry is last in sorted order
-        pkg.set('z_badpath', 'bar')
-        with pytest.raises(QuiltException):
-            pkg.get()
+        Package.rollback('quilt/tmp', LOCAL_REGISTRY, good_hash)
 
-        # package with inconsistent schemes
-        with patch('quilt3.packages.get_size_and_meta', return_value=(0, dict(), '0')):
-            pkg = quilt3.Package().set_dir("/", "./")
-            pkg.set("bar", "s3://test-bucket/bar")
-            with pytest.raises(QuiltException):
-                pkg.get()
+        assert 'foo' in Package.browse('quilt/tmp')
 
-        # package with inconsistent root directories
-        with open('foo_dir/foo', 'w') as fd:
-            fd.write(fd.name)
-        pkg = quilt3.Package().set_dir("/", "./")
-        pkg.set('foo', 'foo_dir/foo')
-        with pytest.raises(QuiltException):
-            pkg.get()
+        with self.assertRaises(QuiltException):
+            Package.rollback('quilt/tmp', LOCAL_REGISTRY, '12345')
+
+        with self.assertRaises(QuiltException):
+            Package.rollback('quilt/blah', LOCAL_REGISTRY, good_hash)
